@@ -1,4 +1,4 @@
-/*      $Id: events.h 19935 2006-02-18 08:21:05Z olivier $
+/*      $Id: events.h 21549 2006-05-06 13:12:18Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -32,12 +32,12 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include <glib.h>
-#include <libxfcegui4/libxfcegui4.h>
 
 #include "screen.h"
+#include "event_filter.h"
 
 void initMenuEventWin (void);
-XfceFilterStatus xfwm4_event_filter (XEvent * xevent, gpointer data);
+eventFilterStatus xfwm4_event_filter (XEvent * xevent, gpointer data);
 void initGtkCallbacks (ScreenInfo *);
 
 #endif /* INC_EVENTS_H */
