@@ -1,4 +1,4 @@
-/*      $Id: focus.c 14226 2005-05-08 21:16:28Z olivier $
+/*      $Id: focus.c 22223 2006-06-30 09:24:40Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
  
@@ -33,7 +33,6 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <libxfce4util/libxfce4util.h> 
-#include <libxfcegui4/libxfcegui4.h>
 
 #include "screen.h"
 #include "focus.h"
@@ -61,8 +60,8 @@ static ClientPair
 clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
 {
     ClientPair top_client;
-    Client *c = NULL;
-    GList *index = NULL;
+    Client *c;
+    GList *index;
 
     TRACE ("entering clientGetTopMostFocusable");
 
@@ -73,8 +72,7 @@ clientGetTopMostFocusable (ScreenInfo *screen_info, int layer, Client * exclude)
         TRACE ("*** stack window \"%s\" (0x%lx), layer %i", c->name,
             c->window, (int) c->win_layer);
 
-        if ((c->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP))
-            || ((layer != WIN_LAYER_DESKTOP) && (c->type & WINDOW_DESKTOP)))
+        if (!clientAcceptFocus (c) || (c->type & (WINDOW_SPLASHSCREEN | WINDOW_DOCK | WINDOW_DESKTOP)))
         {
             continue;
         }
@@ -103,7 +101,7 @@ void
 clientFocusTop (ScreenInfo *screen_info, int layer)
 {
     ClientPair top_client;
-    DisplayInfo *display_info = NULL;
+    DisplayInfo *display_info;
 
     display_info = screen_info->display_info;
     top_client = clientGetTopMostFocusable (screen_info, layer, NULL);
@@ -121,49 +119,56 @@ clientFocusTop (ScreenInfo *screen_info, int layer)
     }
 }
 
-void
+gboolean
 clientFocusNew(Client * c)
 {
-    ScreenInfo *screen_info = NULL;
-    DisplayInfo *display_info = NULL;
+    ScreenInfo *screen_info;
+    DisplayInfo *display_info;
     gboolean give_focus;
     gboolean prevent_focus_stealing;
+    gboolean prevented;
 
-    g_return_if_fail (c != NULL);
+    g_return_val_if_fail (c != NULL, FALSE);
     
-    if (!clientAcceptFocus (c)|| (c->type & WINDOW_TYPE_DONT_FOCUS))
-    {
-        return;
-    }
     screen_info = c->screen_info;
     display_info = screen_info->display_info;
-    give_focus = screen_info->params->focus_new;
+    give_focus = (c-> type & WINDOW_REGULAR_FOCUSABLE) && (screen_info->params->focus_new);
     prevent_focus_stealing = screen_info->params->prevent_focus_stealing;
-
+    prevented = FALSE;
+    
     /*  Try to avoid focus stealing */
-    if ((client_focus) && (prevent_focus_stealing))
+    if (!clientAcceptFocus (c) || (c->type & WINDOW_TYPE_DONT_FOCUS))
     {
-        if (FLAG_TEST(c->flags, CLIENT_FLAG_HAS_USER_TIME) &&
-            FLAG_TEST(client_focus->flags, CLIENT_FLAG_HAS_USER_TIME))
+        give_focus = FALSE;
+    }
+    else if ((client_focus) && (prevent_focus_stealing))
+    {
+        if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME) && (c->user_time == (Time) 0))
+        {
+            TRACE ("Given startup time is 0, not focusing");
+            give_focus = FALSE;
+            prevented = TRUE;
+        }
+        else if (FLAG_TEST (c->flags, CLIENT_FLAG_HAS_STARTUP_TIME | CLIENT_FLAG_HAS_USER_TIME))
         {
-            TRACE ("Current %u, new %u", client_focus->user_time, c->user_time);
             if (c->user_time < client_focus->user_time)
             {
+                TRACE ("Current %u, new %u", (unsigned int) client_focus->user_time, (unsigned int) c->user_time);
                 give_focus = FALSE;
+                prevented = TRUE;
             }
         }
     }
 
-    clientGrabMouseButton (c);
-    if (give_focus || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
+    if ((give_focus) || FLAG_TEST(c->flags, CLIENT_FLAG_STATE_MODAL))
     {
-        if (client_focus)
+        give_focus = TRUE;
+        if ((client_focus) && !(clientIsTransientOrModalFor (c, client_focus)))
         {
-            if (clientAdjustFullscreenLayer (client_focus, FALSE))
-            {
-                clientRaise (c);
-            }
+            clientAdjustFullscreenLayer (client_focus, FALSE);
         }
+        clientRaise (c, None);
+        clientShow (c, TRUE);
         clientSetFocus (screen_info, c, 
 #if 0
                         myDisplayGetCurrentTime (display_info), 
@@ -174,10 +179,28 @@ clientFocusNew(Client * c)
     }
     else
     {
-        FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+        Client *c2 = clientGetFocus();
+
+        if (prevented && c2 && (c2->win_layer == c->win_layer))
+        {
+            FLAG_SET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSortRing(c);
+            clientLower (c, c2->frame);
+            clientSortRing(c2);
+            clientSetOpacity (c, c->opacity, 0, 0);
+        }
+        else
+        {
+            clientRaise (c, None);
+            clientSortRing(c);
+        }
+
+        clientShow (c, TRUE);
         clientSetNetState (c);
         clientGrabMouseButton (c);
     }
+
+    return (give_focus);
 }
 
 gboolean
@@ -220,7 +243,7 @@ clientSelectMask (Client * c, int mask, int type)
 Client *
 clientGetNext (Client * c, int mask)
 {
-    Client *c2 = NULL;
+    Client *c2;
     unsigned int i;
 
     TRACE ("entering clientGetNext");
@@ -243,7 +266,7 @@ clientGetNext (Client * c, int mask)
 Client *
 clientGetPrevious (Client * c, int mask)
 {
-    Client *c2 = NULL;
+    Client *c2;
     unsigned int i;
 
     TRACE ("entering clientGetPrevious");
@@ -266,18 +289,23 @@ clientGetPrevious (Client * c, int mask)
 void
 clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
 {
-    DisplayInfo *display_info = NULL;
-    Client *new_focus = NULL;
-    Client *current_focus = client_focus;
+    DisplayInfo *display_info;
     ClientPair top_most;
-    Client *c2 = NULL;
+    Client *new_focus;
+    Client *current_focus;
+    Client *c2;
     Window dr, window;
-    int rx, ry, wx, wy;
     unsigned int mask;
-    int look_in_layer = (c ? c->win_layer : WIN_LAYER_NORMAL);
+    int rx, ry, wx, wy;
+    int look_in_layer;
 
     TRACE ("entering clientPassFocus");
 
+    look_in_layer = (c ? c->win_layer : WIN_LAYER_NORMAL);
+    new_focus = NULL;
+    current_focus = client_focus;
+    c2 = NULL;
+
     if (pending_focus)
     {
         current_focus = pending_focus;
@@ -288,6 +316,11 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
         return;
     }
 
+    if (current_focus == last_ungrab)
+    {
+        clientPassGrabMouseButton (NULL);
+    }
+
     display_info = screen_info->display_info;
     top_most = clientGetTopMostFocusable (screen_info, look_in_layer, exclude);
     if (screen_info->params->click_to_focus)
@@ -296,8 +329,8 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
         {
             if (clientIsModal (c))
             {
-                /* If the window is a modal, send focus back to its parent window
-                   Modals are transients, and we aren"t interested in modal
+                /* If the window is a modal, send focus back to its parent window.
+                   Modals are transients, and we aren't interested in modal
                    for group, so it safe to use clientGetTransient because 
                    it's really what we want...
                  */
@@ -306,7 +339,7 @@ clientPassFocus (ScreenInfo *screen_info, Client *c, Client *exclude)
                 {
                     new_focus = c2;
                     /* Usability: raise the parent, to grab user's attention */
-                    clientRaise (c2);
+                    clientRaise (c2, None);
                 }
             }
             else
@@ -353,24 +386,25 @@ clientAcceptFocus (Client * c)
     {
         return FALSE;
     }
-    if (!FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
+    if ((c->screen_info->params->focus_hint) 
+        && !FLAG_TEST (c->wm_flags, WM_FLAG_INPUT | WM_FLAG_TAKEFOCUS))
     {
         return FALSE;
     }
-    
+
     return TRUE;
 }
 
 void
 clientSortRing(Client *c)
 {
-    ScreenInfo *screen_info = NULL;
+    ScreenInfo *screen_info;
     
     g_return_if_fail (c != NULL);
 
     TRACE ("Sorting...");
     screen_info = c->screen_info;
-    if (screen_info->client_count > 2 && c != screen_info->clients)
+    if ((screen_info->client_count > 2) && (c != screen_info->clients)) 
     {
         c->prev->next = c->next;
         c->next->prev = c->prev;
@@ -386,12 +420,13 @@ clientSortRing(Client *c)
 void
 clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
 {
-    DisplayInfo *display_info = NULL;
-    Client *c2 = ((client_focus != c) ? client_focus : NULL);
+    DisplayInfo *display_info;
+    Client *c2;
     unsigned long data[2];
 
     TRACE ("entering clientUpdateFocus");
 
+    c2 = ((client_focus != c) ? client_focus : NULL);
     display_info = screen_info->display_info;
     pending_focus = NULL;
     if ((c) && !clientAcceptFocus (c))
@@ -406,7 +441,11 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         return;
     }
 
-    if (c == clientGetLastRaise (screen_info))
+    /* 
+       We can release the button mouse grab if we don't raise on click or if the focused window
+       is the one that has been raised at last.
+     */
+    if (!(screen_info->params->raise_on_click) || (c == clientGetLastRaise (screen_info)))
     {
         clientPassGrabMouseButton (c);
     }
@@ -432,7 +471,7 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
         if (c)
         {
             clientAdjustFullscreenLayer (c2, FALSE);
-            /* clientRaise (c); */
+            /* clientRaise (c, None); */
         }
         frameDraw (c2, FALSE, FALSE);
     }
@@ -440,17 +479,20 @@ clientUpdateFocus (ScreenInfo *screen_info, Client * c, unsigned short flags)
     XChangeProperty (display_info->dpy, screen_info->xroot, 
                      display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                      PropModeReplace, (unsigned char *) data, 2);
+
+    clientUpdateOpacity (screen_info, c);
 }
 
 void
 clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned short flags)
 {
-    DisplayInfo *display_info = NULL;
-    Client *c2 = NULL;
+    DisplayInfo *display_info;
+    Client *c2;
 
     TRACE ("entering clientSetFocus");
     
     display_info = screen_info->display_info;
+    c2 = NULL;
     if ((c) && !(flags & FOCUS_IGNORE_MODAL))
     {
         c2 = clientGetModalFor (c);
@@ -464,6 +506,11 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
     if ((c) && FLAG_TEST (c->xfwm_flags, XFWM_FLAG_VISIBLE))
     {
         TRACE ("setting focus to client \"%s\" (0x%lx)", c->name, c->window);
+        if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
+        {
+            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
+            clientSetNetState (c);
+        }
         if ((c == client_focus) && !(flags & FOCUS_FORCE))
         {
             TRACE ("client \"%s\" (0x%lx) is already focused, ignoring request",
@@ -475,20 +522,16 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
             TRACE ("SKIP_FOCUS set for client \"%s\" (0x%lx)", c->name, c->window);
             return;
         }
-        if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT))
+        if (FLAG_TEST (c->wm_flags, WM_FLAG_INPUT) || !(screen_info->params->focus_hint))
         {
             pending_focus = c;
             XSetInputFocus (myScreenGetXDisplay (screen_info), c->window, RevertToPointerRoot, timestamp);
+            clientUpdateOpacity (screen_info, c);
         }
         if (FLAG_TEST(c->wm_flags, WM_FLAG_TAKEFOCUS))
         {
             sendClientMessage (c->screen_info, c->window, WM_TAKE_FOCUS, timestamp);
         }
-        if (FLAG_TEST(c->flags, CLIENT_FLAG_DEMANDS_ATTENTION))
-        {
-            FLAG_UNSET (c->flags, CLIENT_FLAG_DEMANDS_ATTENTION);
-            clientSetNetState (c);
-        }
     }
     else
     {
@@ -506,7 +549,8 @@ clientSetFocus (ScreenInfo *screen_info, Client * c, Time timestamp, unsigned sh
         }
         XChangeProperty (myScreenGetXDisplay (screen_info), screen_info->xroot, display_info->atoms[NET_ACTIVE_WINDOW], XA_WINDOW, 32,
                          PropModeReplace, (unsigned char *) data, 2);
-        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->gnome_win, RevertToPointerRoot, timestamp);
+        XSetInputFocus (myScreenGetXDisplay (screen_info), screen_info->xfwm4_win, RevertToPointerRoot, timestamp);
+        clientUpdateOpacity (screen_info, c);
     }
 }
 
@@ -525,7 +569,7 @@ clientClearFocus (void)
 void
 clientGrabMouseButton (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
+    ScreenInfo *screen_info;
     
     g_return_if_fail (c != NULL);
     TRACE ("entering clientGrabMouseButton");
@@ -534,18 +578,18 @@ clientGrabMouseButton (Client * c)
     screen_info = c->screen_info;
     if (screen_info->params->raise_with_any_button)
     {
-        grabButton(clientGetXDisplay (c), AnyButton, 0, c->window);
+        grabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
     }
     else
     {
-        grabButton(clientGetXDisplay (c), Button1, 0, c->window);
+        grabButton(clientGetXDisplay (c), Button1, AnyModifier, c->window);
     }
 }
 
 void
 clientUngrabMouseButton (Client * c)
 {
-    ScreenInfo *screen_info = NULL;
+    ScreenInfo *screen_info;
     
     g_return_if_fail (c != NULL);
     TRACE ("entering clientUngrabMouseButton");
@@ -554,12 +598,14 @@ clientUngrabMouseButton (Client * c)
     screen_info = c->screen_info;
     if (screen_info->params->raise_with_any_button)
     {
-        ungrabButton(clientGetXDisplay (c), AnyButton, 0, c->window);
+        ungrabButton(clientGetXDisplay (c), AnyButton, AnyModifier, c->window);
     }
     else
     {
-        ungrabButton(clientGetXDisplay (c), Button1, 0, c->window);
+        ungrabButton(clientGetXDisplay (c), Button1, AnyModifier, c->window);
     }
+    /* We've ungrabbed way too much, so regrab the regular buttons/modifiers */
+    clientGrabButtons (c);
 }
 
 void
@@ -598,7 +644,6 @@ void
 clientPassGrabMouseButton (Client * c)
 {
     TRACE ("entering clientPassMouseGrabButton");
-    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
 
     if (c == NULL)
     {
@@ -610,6 +655,7 @@ clientPassGrabMouseButton (Client * c)
         return;
     }
 
+    TRACE ("ungrabing buttons for client \"%s\" (0x%lx)", c->name, c->window);
     if (last_ungrab == c)
     {
         return;
