@@ -1,4 +1,4 @@
-/*      $Id: hints.c 20650 2006-03-30 19:17:24Z olivier $
+/*      $Id: hints.c 22243 2006-07-02 16:23:01Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -49,17 +49,57 @@ check_type_and_format (int expected_format, Atom expected_type, int n_items, int
     return FALSE;
 }
 
+static gchar *
+internal_utf8_strndup (const gchar *src, gssize max_len)
+{
+    const gchar *s;
+
+    if (max_len <= 0)
+    {
+        return g_strdup (src);
+    }
+
+    s = src;
+    while (max_len && *s)
+    {
+        s = g_utf8_next_char (s);
+        max_len--;
+    }
+
+    return g_strndup (src, s - src);
+}
+
+static gchar*
+create_name_with_host (DisplayInfo *display_info, const gchar *name, const gchar *hostname)
+{
+    gchar *title;
+    
+    if (strlen (hostname) && (display_info->hostname) && (g_strcasecmp (display_info->hostname, hostname)))
+    {
+        /* TRANSLATORS: "(on %s)" is like "running on" the name of the other host */
+        title = g_strdup_printf (_("%s (on %s)"), name, hostname);
+    }
+    else
+    {
+        title = g_strdup (name);        
+    }
+
+    return title;
+}
+
 unsigned long
 getWMState (DisplayInfo *display_info, Window w)
 {
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    unsigned char *data = NULL;
-    unsigned long state = WithdrawnState;
+    unsigned char *data;
+    unsigned long state;
 
     TRACE ("entering getWmState");
 
+    data = NULL;
+    state = WithdrawnState;
     if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[WM_STATE], 
                              0, 3L, FALSE, display_info->atoms[WM_STATE],
                              &real_type, &real_format, &items_read, &items_left,
@@ -95,11 +135,13 @@ getMotifHints (DisplayInfo *display_info, Window w)
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    unsigned char *data = NULL;
-    PropMwmHints *result = NULL;
+    unsigned char *data;
+    PropMwmHints *result;
 
     TRACE ("entering getMotifHints");
 
+    data = NULL;
+    result = NULL;
     if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[MOTIF_WM_HINTS], 0L, MWM_HINTS_ELEMENTS, 
                 FALSE, display_info->atoms[MOTIF_WM_HINTS], &real_type, &real_format, &items_read,
                 &items_left, (unsigned char **) &data) == Success))
@@ -124,12 +166,13 @@ getWMProtocols (DisplayInfo *display_info, Window w)
     int i, n;
     Atom atype;
     int aformat;
-    unsigned int result = 0;
+    unsigned int result;
     unsigned long bytes_remain, nitems;
     unsigned char *data;
 
     TRACE ("entering getWMProtocols");
 
+    result = 0;
     if (XGetWMProtocols (display_info->dpy, w, &protocols, &n))
     {
         for (i = 0, ap = protocols; i < n; i++, ap++)
@@ -190,15 +233,17 @@ gboolean
 getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
 {
     Atom real_type;
-    int real_format;
-    gboolean success = FALSE;
     unsigned long items_read, items_left;
-    unsigned char *data = NULL;
+    unsigned char *data;
+    int real_format;
+    gboolean success;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     TRACE ("entering getHint");
 
+    success = FALSE;
     *value = 0;
+    data = NULL;
 
     if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 0L, 1L, 
                              FALSE, XA_CARDINAL, &real_type, &real_format, &items_read, &items_left,
@@ -217,7 +262,7 @@ getHint (DisplayInfo *display_info, Window w, int atom_id, long *value)
 void
 setHint (DisplayInfo *display_info, Window w, int atom_id, long value)
 {
-    g_return_if_fail ((atom_id > 0) && (atom_id < NB_ATOMS));
+    g_return_if_fail ((atom_id >= 0) && (atom_id < NB_ATOMS));
     TRACE ("entering setHint");
 
     XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], XA_CARDINAL, 
@@ -228,15 +273,19 @@ void
 getDesktopLayout (DisplayInfo *display_info, Window root, int ws_count, NetWmDesktopLayout * layout)
 {
     Atom real_type;
-    int real_format;
-    gboolean success = FALSE;
     unsigned long items_read, items_left;
     unsigned long orientation, cols, rows, start;
-    unsigned long *ptr = NULL;
-    unsigned char *data = NULL;
+    unsigned long *ptr;
+    unsigned char *data;
+    int real_format;
+    gboolean success;
 
     TRACE ("entering getDesktopLayout");
 
+    ptr = NULL;
+    data = NULL;
+    success = FALSE;
+
     if ((XGetWindowProperty (display_info->dpy, root, display_info->atoms[NET_DESKTOP_LAYOUT],
                 0L, 4L, FALSE, XA_CARDINAL,
                 &real_type, &real_format, &items_read, &items_left,
@@ -301,11 +350,13 @@ getGnomeDesktopMargins (DisplayInfo *display_info, Window root, int * m)
     Atom real_type;
     int real_format;
     unsigned long items_read, items_left;
-    unsigned long *ptr = NULL;
-    unsigned char *data = NULL;
+    unsigned long *ptr;
+    unsigned char *data;
 
     TRACE ("entering getGnomeDesktopMargins");
 
+    ptr = NULL;
+    data = NULL;
     if ((XGetWindowProperty (display_info->dpy, root,
                 display_info->atoms[GNOME_PANEL_DESKTOP_AREA], 0L, 4L, FALSE, XA_CARDINAL,
                 &real_type, &real_format, &items_read, &items_left,
@@ -344,8 +395,9 @@ setNetSupportedHint (DisplayInfo *display_info, Window root, Window check_win)
 {
     Atom atoms[64];
     unsigned long data[1];
-    int i = 0;
+    int i;
 
+    i = 0;
     atoms[i++] = display_info->atoms[NET_ACTIVE_WINDOW];
     atoms[i++] = display_info->atoms[NET_CLIENT_LIST];
     atoms[i++] = display_info->atoms[NET_CLIENT_LIST_STACKING];
@@ -427,7 +479,7 @@ getAtomList (DisplayInfo *display_info, Window w, int atom_id, Atom ** atoms_p,
     *atoms_p = NULL;
     *n_atoms_p = 0;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     TRACE ("entering getAtomList()");
 
     if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
@@ -467,7 +519,7 @@ getCardinalList (DisplayInfo *display_info, Window w, int atom_id, unsigned long
     *cardinals_p = NULL;
     *n_cardinals_p = 0;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     TRACE ("entering getCardinalList()");
 
     if ((XGetWindowProperty (display_info->dpy, w, display_info->atoms[atom_id], 
@@ -545,9 +597,9 @@ initNetDesktopInfo (DisplayInfo *display_info, Window root, int workspace, int w
 }
 
 void
-setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const char *val)
+setUTF8StringHint (DisplayInfo *display_info, Window w, int atom_id, const gchar *val)
 {
-    g_return_if_fail ((atom_id > 0) && (atom_id < NB_ATOMS));
+    g_return_if_fail ((atom_id >= 0) && (atom_id < NB_ATOMS));
     TRACE ("entering setUTF8StringHint");
 
     XChangeProperty (display_info->dpy, w, display_info->atoms[atom_id], 
@@ -581,23 +633,6 @@ getTransientFor (DisplayInfo *display_info, Window root, Window w, Window * tran
     TRACE ("Window (0x%lx) is transient for (0x%lx)", w, *transient_for);
 }
 
-static gchar *
-internal_utf8_strndup (const gchar *src, gssize max_len)
-{
-    const gchar *s = src;
-
-    if (max_len <= 0)
-        return g_strdup (src);
-
-    while (max_len && *s)
-    {
-        s = g_utf8_next_char (s);
-        max_len--;
-    }
-
-    return g_strndup (src, s - src);
-}
-
 static char *
 text_property_to_utf8 (DisplayInfo *display_info, const XTextProperty * prop)
 {
@@ -652,7 +687,7 @@ get_text_property (DisplayInfo *display_info, Window w, Atom a)
 }
 
 static gboolean
-getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p, int *length)
 {
     Atom type;
     int format;
@@ -660,7 +695,7 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_
     unsigned char *str;
     unsigned long n_items;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     TRACE ("entering getUTF8StringData");
 
     *str_p = NULL;
@@ -689,11 +724,11 @@ getUTF8StringData (DisplayInfo *display_info, Window w, int atom_id, char **str_
 }
 
 gboolean
-getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, int *length)
+getUTF8String (DisplayInfo *display_info, Window w, int atom_id, gchar **str_p, int *length)
 {
     char *xstr;
     
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     TRACE ("entering getUTF8String");
 
     if (!getUTF8StringData (display_info, w, atom_id, &xstr, length))
@@ -727,14 +762,14 @@ getUTF8String (DisplayInfo *display_info, Window w, int atom_id, char **str_p, i
 }
 
 gboolean
-getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str_p, int *n_items)
+getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, gchar ***str_p, int *n_items)
 {
     char *xstr, *ptr;
-    char **retval;
+    gchar **retval;
     guint i;
     int length;
 
-    g_return_val_if_fail (((atom_id > 0) && (atom_id < NB_ATOMS)), FALSE);
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
 
     TRACE ("entering getUTF8StringList");
 
@@ -760,7 +795,7 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str
         *n_items = *n_items + 1;
     }
  
-    retval = g_new0 (char *, *n_items + 1);
+    retval = g_new0 (gchar *, *n_items + 1);
     ptr = xstr;
     
     for (i = 0; i < *n_items; i++)
@@ -782,38 +817,69 @@ getUTF8StringList (DisplayInfo *display_info, Window w, int atom_id, char ***str
     return TRUE;
 }
 
-void
-getWindowName (DisplayInfo *display_info, Window w, char **name)
+gboolean
+getClientMachine (DisplayInfo *display_info, Window w, gchar **machine)
 {
     char *str;
-    int len;
+    gboolean status;
 
-    TRACE ("entering getWindowName");
+    TRACE ("entering getClientMachine");
 
-    g_return_if_fail (name != NULL);
-    *name = NULL;
-    g_return_if_fail (w != None);
+    g_return_val_if_fail (machine != NULL, FALSE);
+    *machine = NULL;
+    g_return_val_if_fail (w != None, FALSE);
 
-    if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len))
-    {
-        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
-        XFree (str);
-        return;
-    }
-    str = get_text_property (display_info, w, XA_WM_NAME);
+    status = FALSE;
+    str = get_text_property (display_info, w, display_info->atoms[WM_CLIENT_MACHINE]);
     if (str)
     {
-        *name = internal_utf8_strndup (str, MAX_STR_LENGTH);
+        *machine = g_strndup (str, MAX_STR_LENGTH);
         XFree (str);
+        status = TRUE;
     }
     else
     {
-        *name = strdup ("");
+        *machine = g_strdup ("");
     }
+    return status;
 }
 
 gboolean
-getWindowRole (DisplayInfo *display_info, Window window, char **role)
+getWindowName (DisplayInfo *display_info, Window w, gchar **title)
+{
+    char *str;
+    int len;
+    gchar *machine;
+    gchar *name;
+    gboolean status;
+    
+    TRACE ("entering getWindowName");
+
+    g_return_val_if_fail (title != NULL, FALSE);
+    *title = NULL;
+    g_return_val_if_fail (w != None, FALSE);
+
+    status = FALSE;
+    getClientMachine (display_info, w, &machine);
+    if (getUTF8StringData (display_info, w, NET_WM_NAME, &str, &len) ||
+        (str = get_text_property (display_info, w, XA_WM_NAME)))
+    {
+        name = internal_utf8_strndup (str, MAX_STR_LENGTH);
+        *title = create_name_with_host (display_info, name, machine);
+        g_free (name);
+        XFree (str);
+        status = TRUE;
+    }
+    else
+    {
+        *title = g_strdup ("");
+    }
+    g_free (machine);
+    return status;
+}
+
+gboolean
+getWindowRole (DisplayInfo *display_info, Window window, gchar **role)
 {
     XTextProperty tp;
 
@@ -829,7 +895,7 @@ getWindowRole (DisplayInfo *display_info, Window window, char **role)
         {
             if ((tp.encoding == XA_STRING) && (tp.format == 8) && (tp.nitems != 0))
             {
-                *role = strdup ((char *) tp.value);
+                *role = g_strdup ((gchar *) tp.value);
                 XFree (tp.value);
                 return TRUE;
             }
@@ -843,7 +909,7 @@ getWindowRole (DisplayInfo *display_info, Window window, char **role)
 Window
 getClientLeader (DisplayInfo *display_info, Window window)
 {
-    Window client_leader = None;
+    Window client_leader;
     Atom actual_type;
     int actual_format;
     unsigned long nitems;
@@ -854,6 +920,7 @@ getClientLeader (DisplayInfo *display_info, Window window)
 
     g_return_val_if_fail (window != None, None);
 
+    client_leader = None;
     if (XGetWindowProperty (display_info->dpy, window, display_info->atoms[WM_CLIENT_LEADER], 
                             0L, 1L, FALSE, AnyPropertyType, &actual_type, &actual_format, &nitems,
                             &bytes_after, (unsigned char **) &prop) == Success)
@@ -901,7 +968,7 @@ getNetWMUserTime (DisplayInfo *display_info, Window window, Time *time)
 }
 
 gboolean
-getClientID (DisplayInfo *display_info, Window window, char **client_id)
+getClientID (DisplayInfo *display_info, Window window, gchar **client_id)
 {
     Window id;
     XTextProperty tp;
@@ -918,7 +985,7 @@ getClientID (DisplayInfo *display_info, Window window, char **client_id)
         {
             if (tp.encoding == XA_STRING && tp.format == 8 && tp.nitems != 0)
             {
-                *client_id = strdup ((char *) tp.value);
+                *client_id = g_strdup ((gchar *) tp.value);
                 XFree (tp.value);
                 return TRUE;
             }
@@ -1019,12 +1086,13 @@ getRGBIconData (DisplayInfo *display_info, Window window, unsigned long **data,
 gboolean
 getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
 {
-    long val = 0;
+    long val;
     
     g_return_val_if_fail (window != None, FALSE);
     g_return_val_if_fail (opacity != NULL, FALSE);
     TRACE ("entering getOpacity");    
 
+    val = 0;
     if (getHint (display_info, window, NET_WM_WINDOW_OPACITY, &val))
     {
         *opacity = (guint) val;
@@ -1035,15 +1103,28 @@ getOpacity (DisplayInfo *display_info, Window window, guint *opacity)
 }
 
 gboolean
-setCompositingManagerOwner (DisplayInfo *display_info, Window root, Window w)
+getOpacityLock (DisplayInfo *display_info, Window window)
+{
+    long val;
+
+    g_return_val_if_fail (window != None, FALSE);
+    TRACE ("entering getOpacityLock");
+
+    /* only presence/absence matters */
+    return !!getHint (display_info, window, NET_WM_WINDOW_OPACITY_LOCKED, &val);
+}
+
+gboolean
+setAtomManagerOwner (DisplayInfo *display_info, int atom_id, Window root, Window w)
 {
     XClientMessageEvent ev;
     Time server_time;
     int status;
     
+    g_return_val_if_fail (((atom_id >= 0) && (atom_id < NB_ATOMS)), FALSE);
     server_time = myDisplayGetCurrentTime (display_info);
     status = XSetSelectionOwner (display_info->dpy, 
-                                 display_info->atoms[COMPOSITING_MANAGER],
+                                 display_info->atoms[atom_id],
                                  w, server_time);
 
     if ((status == BadAtom) || (status == BadWindow))
@@ -1051,13 +1132,13 @@ setCompositingManagerOwner (DisplayInfo *display_info, Window root, Window w)
         return FALSE;
     }
 
-    if (XGetSelectionOwner (display_info->dpy, display_info->atoms[COMPOSITING_MANAGER]) == w)
+    if (XGetSelectionOwner (display_info->dpy, display_info->atoms[atom_id]) == w)
     {
         ev.type = ClientMessage;
-        ev.message_type = display_info->atoms[COMPOSITING_MANAGER];
+        ev.message_type = display_info->atoms[atom_id];
         ev.format = 32;
         ev.data.l[0] = server_time;
-        ev.data.l[1] = display_info->atoms[COMPOSITING_MANAGER];
+        ev.data.l[1] = display_info->atoms[atom_id];
         ev.data.l[2] = w;
         ev.data.l[3] = 0;
         ev.data.l[4] = 0;
@@ -1122,7 +1203,7 @@ sendSystrayReqDock(DisplayInfo *display_info, Window window, Window systray)
 Window
 getSystrayWindow (DisplayInfo *display_info, Atom net_system_tray_selection)
 {
-    Window systray_win = None;
+    Window systray_win;
 
     TRACE ("entering getSystrayWindow");
 
@@ -1138,7 +1219,7 @@ getSystrayWindow (DisplayInfo *display_info, Atom net_system_tray_selection)
 
 #ifdef HAVE_LIBSTARTUP_NOTIFICATION
 gboolean
-getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
+getWindowStartupId (DisplayInfo *display_info, Window w, gchar **startup_id)
 {
     char *str;
     int len;
@@ -1151,7 +1232,7 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
 
     if (getUTF8StringData (display_info, w, NET_STARTUP_ID, &str, &len))
     {
-        *startup_id = strdup (str);
+        *startup_id = g_strdup (str);
         XFree (str);
         return TRUE;
     }
@@ -1159,7 +1240,7 @@ getWindowStartupId (DisplayInfo *display_info, Window w, char **startup_id)
     str = get_text_property (display_info, w, NET_STARTUP_ID);
     if (str)
     {
-        *startup_id = strdup (str);
+        *startup_id = g_strdup (str);
         XFree (str);
         return TRUE;
     }
