@@ -1,4 +1,4 @@
-/*      $Id: icons.h 12159 2004-11-23 23:57:33Z olivier $
+/*      $Id: icons.h 21549 2006-05-06 13:12:18Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  
         Metacity - (c) 2001 Havoc Pennington
         libwnck  - (c) 2001 Havoc Pennington
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,6 +32,9 @@
 
 #include "display.h"
 
-GdkPixbuf *getAppIcon (DisplayInfo *, Window, int, int);
+GdkPixbuf * getAppIcon          (DisplayInfo *, 
+                                 Window, 
+                                 int, 
+                                 int);
 
 #endif /* INC_ICONS_H */
