@@ -1,4 +1,4 @@
-/*      $Id: misc.h 12192 2004-12-09 22:44:20Z olivier $
+/*      $Id: misc.h 22213 2006-06-28 21:18:22Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
         oroborus - (c) 2001 Ken Lynch
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  
  */
 
@@ -31,10 +31,23 @@
 #ifndef INC_MISC_H
 #define INC_MISC_H
 
+/* 
+   Just for completion, being a bit pedantic, X seems to be missing 
+   those so far, so create them for now.
+ */
+#ifndef Button6
+#define Button6 6
+#endif
+
+#ifndef Button7
+#define Button7 7
+#endif
+
 void getMouseXY (ScreenInfo *, Window, int *, int *);
 Window getMouseWindow (ScreenInfo *, Window);
 GC createGC (ScreenInfo *, char *, int, XFontStruct *, int, gboolean);
 void sendClientMessage (ScreenInfo *, Window, int, Time);
+void sendRootMessage (ScreenInfo *, int, long, Time);
 gboolean checkWindowOnRoot (ScreenInfo *, Window);
 void placeSidewalks(ScreenInfo *, gboolean);
 gint find_monitor_at_point (GdkScreen *screen, gint x, gint y);
