@@ -1,4 +1,4 @@
-/*      $Id: mywindow.c 20674 2006-04-01 20:38:05Z olivier $
+/*      $Id: mywindow.c 21694 2006-05-15 21:57:22Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -70,7 +70,9 @@ xfwmWindowInit (xfwmWindow * win)
     win->y = 0;
     win->width = 1;
     win->height = 1;
+#ifdef HAVE_RENDER
     win->pict_format = NULL;
+#endif
 }
 
 void
@@ -290,13 +292,14 @@ xfwmWindowCopyComposite (xfwmWindow * win, xfwmPixmap * pix)
 void
 xfwmWindowSetBG (xfwmWindow * win, xfwmPixmap * pix)
 {
-    gboolean done = FALSE;
-    
+    gboolean done;
+
     if ((win->width < 1) || (win->height < 1) || (pix->width < 1) || (pix->height < 1))
     {
         return;
     }
 
+    done = FALSE;
 #ifdef HAVE_RENDER
     if ((win->visual != win->screen_info->visual) || 
         (win->depth  != win->screen_info->depth))
