@@ -1,4 +1,4 @@
-/*      $Id: parserc.c 19935 2006-02-18 08:21:05Z olivier $
+/*      $Id: parserc.c 21694 2006-05-15 21:57:22Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -81,10 +81,11 @@ gboolean
 checkRc (Settings *rc)
 {
     gint i;
-    gboolean rval = TRUE;
+    gboolean rval;
 
     TRACE ("entering checkRc");
 
+    rval = TRUE;
     for (i = 0; rc[i].option; i++)
     {
         if (rc[i].required && !rc[i].value)
@@ -201,7 +202,7 @@ getThemeDir (const gchar * theme, const gchar * file)
         xfce_resource_pop_path (XFCE_RESOURCE_THEMES);
 
         g_free (path);
- 
+
         if (test_file)
         {
             path = g_path_get_dirname (test_file);
