@@ -1,4 +1,4 @@
-/*      $Id: session.h 11914 2004-08-22 22:18:50Z olivier $
+/*      $Id: session.h 21727 2006-05-20 11:33:06Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
         along with this program; if not, write to the Free Software
         Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
-        xfwm4    - (c) 2002-2004 Olivier Fourdan
+        xfwm4    - (c) 2002-2006 Olivier Fourdan
  */
 
 
@@ -27,24 +27,38 @@
 #include "screen.h"
 
 /*
-**  Save window states to file which name is given in argument.
-*/
-gboolean sessionSaveWindowStates (DisplayInfo *, gchar *);
+ *  Save window states to file which name is given in argument.
+ */
+gboolean 
+sessionSaveWindowStates                (DisplayInfo *, 
+                                        gchar *);
 
 /*
-**  Load window states to file which name is given in argument.
-*/
-gboolean sessionLoadWindowStates (gchar *);
+ *  Load window states to file which name is given in argument.
+ */
+gboolean
+sessionLoadWindowStates                (gchar *);
 
 /*
-** Free allocated structure. Should be called before xfwm4 dies
-*/
-void sessionFreeWindowStates (void);
+ * Free allocated structure. Should be called before xfwm4 dies
+ */
+void
+sessionFreeWindowStates                (void);
 
 /*
-** Search for existing client in saved session and update
-** relevant client fields if found.
-*/
-gboolean sessionMatchWinToSM (Client *);
+ * Search for existing client in saved session and update
+ * relevant client fields if found.
+ */
+gboolean
+sessionMatchWinToSM                    (Client *);
+
+/*
+ * Initiate session, connect to session manager and
+ * load saved states if the connection succeeds.
+ */
+int
+sessionStart                           (int, 
+                                        char **,
+                                        DisplayInfo *);
 
 #endif /* INC_CLIENT_H */
