@@ -1,4 +1,4 @@
-/*      $Id: settings.h 20535 2006-03-24 22:55:04Z olivier $
+/*      $Id: settings.h 22306 2006-07-09 09:07:25Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -39,6 +39,10 @@
 #define TITLE_4                         3
 #define TITLE_5                         4
 
+#define TITLE_SHADOW_NONE               0
+#define TITLE_SHADOW_UNDER              1
+#define TITLE_SHADOW_FRAME              2
+
 #define MENU_BUTTON                     0
 #define STICK_BUTTON                    1
 #define SHADE_BUTTON                    2
@@ -113,7 +117,8 @@
 #define KEY_MOVE_DOWN_WORKSPACE         67
 #define KEY_MOVE_LEFT_WORKSPACE         68
 #define KEY_MOVE_RIGHT_WORKSPACE        69
-#define KEY_COUNT                       70
+#define KEY_MOVE_CANCEL                 70
+#define KEY_COUNT                       71
 
 #define ALIGN_LEFT                      0
 #define ALIGN_RIGHT                     1
@@ -148,6 +153,7 @@ struct _XfwmParams
     int double_click_action;
     int easy_click;
     int frame_opacity;
+    int inactive_opacity;
     int move_opacity;
     int placement_ratio;
     int popup_opacity;
@@ -161,8 +167,8 @@ struct _XfwmParams
     int snap_width;
     int title_alignment;
     int title_horizontal_offset;
+    int title_shadow[2];
     int wrap_resistance;
-    gboolean title_shadow[2];
     gboolean box_move;
     gboolean box_resize;
     gboolean click_to_focus;
@@ -186,6 +192,7 @@ struct _XfwmParams
     gboolean title_vertical_offset_active;
     gboolean title_vertical_offset_inactive;
     gboolean toggle_workspaces;
+    gboolean use_compositing;
     gboolean wrap_cycle;
     gboolean wrap_layout;
     gboolean wrap_windows;
