@@ -1,4 +1,4 @@
-/*      $Id: wireframe.c 19935 2006-02-18 08:21:05Z olivier $
+/*      $Id: wireframe.c 21549 2006-05-06 13:12:18Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -27,7 +27,11 @@
 #include <X11/Xutil.h>
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h>
+#include <glib.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <gtk/gtk.h>
+#include <libxfce4util/libxfce4util.h> 
 
 #include "screen.h"
 #include "client.h"
