@@ -1,4 +1,4 @@
-/*      $Id: workspaces.h 20527 2006-03-24 12:26:41Z olivier $
+/*      $Id: workspaces.h 21100 2006-04-19 22:05:43Z olivier $
  
         This program is free software; you can redistribute it and/or modify
         it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 
 gboolean workspaceMove (ScreenInfo *, int, int, Client *);
 void workspaceSwitch (ScreenInfo *, int, Client *, gboolean);
-void workspaceSetNames (ScreenInfo *, char **, int);
+void workspaceSetNames (ScreenInfo *, gchar **, int);
 void workspaceSetCount (ScreenInfo *, int);
 void workspaceUpdateArea (ScreenInfo *);
 
